use crate::{db::ItemSearchParams, static_content::possession_type::possession_data};

use super::{
    get_player_item_or_fail, search_item_for_user, user_error, UResult, UserError, UserVerb,
    UserVerbRef, VerbContext,
};
use ansi::ansi;
use async_trait::async_trait;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        remaining: &str,
    ) -> UResult<()> {
        let player_item = get_player_item_or_fail(ctx).await?;
        if player_item.death_data.is_some() {
            user_error("You can't figure out to turn things on/off whilst dead.".to_owned())?;
        }

        let (state, item_name) = remaining.split_once(" ").ok_or_else(|| {
            UserError(
                ansi!("Try <bold>turn on something<reset> or <bold>turn off something<reset>.")
                    .to_owned(),
            )
        })?;

        let state = state.trim().to_lowercase();
        let to_state = if state == "on" {
            true
        } else if state == "off" {
            false
        } else {
            user_error(
                ansi!("Try <bold>turn on something<reset> or <bold>turn off something<reset>.")
                    .to_owned(),
            )?
        };

        let item_name = item_name.trim();

        let item = search_item_for_user(
            ctx,
            &ItemSearchParams {
                include_contents: true,
                include_loc_contents: true,
                limit: 1,
                ..ItemSearchParams::base(&player_item, item_name)
            },
        )
        .await?;

        let handler = item
            .possession_type
            .as_ref()
            .and_then(|pt| possession_data().get(pt))
            .and_then(|pd| pd.turn_toggle_handler)
            .ok_or_else(|| {
                UserError(format!(
                    "You can't turn that {}!",
                    if to_state { "on" } else { "off" }
                ))
            })?;
        handler.turn_cmd(ctx, &player_item, &item, to_state).await?;

        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
